!----------------------------------------------------------------------
! A module to define all parameters of the run, which can be read from
! command line arguments
module parameters
  use sub_defs_io
  use integration
  use types
  implicit none

  private
  real(dp), parameter, public :: gfermi = 1.16637e-5_dp
  real(dp), parameter, public :: gev2pb = 389379660.0_dp
  real(dp), parameter, public :: eps    = 1.0e-14_dp
  real(dp), public :: xmuf, xmur, Qmin
  real(dp), public :: mh, mh_sq, hwidth, mt, mb
  real(dp), public :: sin_thw, mw, mz, w_width, z_width
  real(dp), public :: sqrts, S, pi, Q0_cut_sq
  real(dp), public :: hmasswindow, toyas
  integer,  public :: order_min, order_max, scale_choice
  integer,  public :: nflav, ipdf, itmx1, itmx2, ncall1, ncall2
  logical,  public :: higgs_use_BW, higgsfixwdth, toypdf
  character * 4, public :: seedstr
  character(len=50), public :: pdfname
  integer, public :: nmempdf
  real(dp), public :: toy_Q0, test_Q0, muR_PDF
  public :: set_parameters

contains

  ! set all parameters from input card or command line arguments
  subroutine set_parameters ()
    real(dp) :: ebeam1, ebeam2
    integer  :: idum, iseed
    common/ranno/idum
    character * 6 WHCPRG
    common/cWHCPRG/WHCPRG
    mw           = dble_val_opt("-mw",80.398_dp)
    mz           = dble_val_opt("-mz",91.187_dp)
    mt           = dble_val_opt("-mt",172.4_dp)
    mb           = dble_val_opt("-mb",4.75_dp)
    nflav        = int_val_opt ("-nf",5)
    mh           = dble_val_opt("-mh",125.0_dp)
    w_width      = dble_val_opt("-wwidth",2.141_dp)
    z_width      = dble_val_opt("-zwidth",2.4952_dp)
    hwidth       = dble_val_opt("-hwidth",0.004029643852284941_dp)
    order_min    = int_val_opt ('-order-min',1)
    order_max    = int_val_opt ('-order-max',4)
    ! if "-lo/-nlo/-nnlo/-n3lo" command is given, overwrite order_min and order_max accordingly
    if (log_val_opt("-lo")) then
       order_min = 1
       order_max = 1
    elseif (log_val_opt("-nlo")) then
       order_min = 1
       order_max = 2
    elseif (log_val_opt("-nnlo")) then
       order_min = 1
       order_max = 3
    elseif (log_val_opt("-n3lo")) then
       order_min = 1
       order_max = 4
    endif    
    sqrts        = dble_val_opt("-sqrts",13000.0_dp)
    scale_choice = int_val_opt ('-scale-choice',1)
    readin       = log_val_opt ("-readingrid")
    higgs_use_BW = log_val_opt ("-higgsbreitwigner")
    hmasswindow  = dble_val_opt("-higgsmasswindow",30.0_dp)
    xmuf         = dble_val_opt("-xmuf",1.0_dp)
    xmur         = dble_val_opt("-xmur",1.0_dp)
    pdfname      = string_val_opt("-pdf", "PDF4LHC15_nnlo_mc")
    nmempdf      = int_val_opt ("-nmempdf",0)
    ncall1       = int_val_opt ("-ncall1",100000)
    ncall2       = int_val_opt ("-ncall2",100000)
    itmx1        = int_val_opt ("-itmx1",12)
    itmx2        = int_val_opt ("-itmx2",12)
    iseed        = int_val_opt ("-iseed",10)
    ! for debugging only:
    toypdf       = log_val_opt("-toy")  ! use a toy PDF
    toy_Q0       = dble_val_opt("-toy-Q0",-1.0_dp) ! Q0 where the initial condition of toy PDF is set 
    toyas        = dble_val_opt("-toy-alphas",-1.0_dp) ! toy alphas value at specified scale
    test_Q0      = dble_val_opt("-dglap-Q0",-1.0_dp) ! Q0 for hoppet DGLAP evolution
    muR_PDF      = dble_val_opt("-muR_PDF",1.0_dp) ! for debugging only

    ! various setup
    write(seedstr,"(I4.4)") iseed
    idum = -iseed
    Qmin = 1.0_dp
    pi   = 4.0_dp*atan(1.0_dp)
    Q0_cut_sq = 4.0_dp
    mh_sq = mh**2
    S = sqrts**2
    ! compute sin(\theta_w) from W/Z mass
    sin_thw = 1.0_dp - (mw/mz)**2

    if (.not.CheckAllArgsUsed(0)) call exit()
  end subroutine set_parameters
  
end module parameters
